/*
** Module   : AppServerEntry.java
** Abstract : Provides low-level APIs to connect to and communicate with an appserver.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20130529 Created initial version.
*/

package com.goldencode.p2j.main;

import com.goldencode.p2j.util.*;

/**
 * Provide low-level APIs to connect and communicate with a remote appserver. This is exported
 * as a network server and statically implemented by the {@link AppServerManager} class.
 * <p>
 * These low-level APIs are meant to be used only by P2J runtime code. For outside access, use 
 * the APIs defined by the {@link AppServerHelper helper} class.
 */
public interface AppServerEntry
{
   /**
    * Establish a new appserver connection. This will invoke any connect procedure for this 
    * appserver, to which the given <code>user</code>, <code>pwd</code> and <code>serverInfo</code>
    * parameters will be passed.
    *  
    * @param    appServer
    *           The name of the target appserver.
    * @param    sessionFree
    *           Flag indicating if the connection request is for a session-free operating mode.
    * @param    user
    *           The first parameter to pass to the connect procedure, if any.
    * @param    pwd
    *           The second parameter to pass to the connect procedure, if any.
    * @param    serverInfo
    *           The third parameter to pass to the connect procedure, if any.
    *           
    * @return   An {@link AppServerInvocationResult} instance with any result or caught errors.
    */
   public AppServerInvocationResult connect(String  appServer,
                                            boolean sessionFree,
                                            String  user,
                                            String  pwd,
                                            String  serverInfo);

   /**
    * Disconnect the appserver connection, with the given ID.  It will invoke any configured 
    * disconnect procedure, but will not return any errors or values to the caller.
    * 
    * @param    id
    *           The connection ID to disconnect.
    */
   public void disconnect(String id);

   /**
    * Invoke a remote external procedure, persistently. The validation of  the program name and
    * arguments will be done on the remote side. Any errors, result or updated arguments will be
    * sent back to the caller.
    * 
    * @param    id
    *           The appserver connection id. 
    * @param    name
    *           The legacy 4GL name for the procedure.
    * @param    transactionDistinct
    *           Flag indicating if the TRANSACTION DISTINCT clause is in effect.
    * @param    modes
    *           A string representation of the modes of each parameter. May be <code>null</code>
    * @param    args
    *           The procedure's arguments.
    *
    * @return   A {@link AppServerInvocationResult} instance, which holds the code for the remote
    *           procedure, all updated OUTPUT/INPUT-OUTPUT arguments and any returned value.
    */
   public AppServerInvocationResult invokePersistent(String    id,
                                                     character name,
                                                     boolean   transactionDistinct,
                                                     String    modes,
                                                     Object... args);

   /**
    * Invoke a remote external procedure, internal procedure or user-defined function. The 
    * validation for the program name and arguments will be done on the remote side. Any errors,
    * result or updated arguments will be sent back to the caller.
    * 
    * @param    id
    *           The appserver connection id. 
    * @param    name
    *           The legacy 4GL name for the procedure/function.
    * @param    resId
    *           The code of a remote procedure in which to invoke the procedure or function.
    * @param    function
    *           <code>true</code> if this is a function call.
    * @param    dynamicFunction
    *           <code>true</code> if this is a DYNAMIC-FUNCTION call.
    * @param    superCall
    *           <code>true</code> if this is a RUN SUPER or SUPER() call.
    * @param    transactionDistinct
    *           Flag indicating if the TRANSACTION DISTINCT clause is in effect.
    * @param    modes
    *           A string representation of the modes of each parameter. May be <code>null</code>
    * @param    args
    *           The procedure's arguments.
    * 
    * @return    A {@link AppServerInvocationResult} instance with all updated OUTPUT/INPUT-OUTPUT
    *            arguments and any returned value.
    */
   public AppServerInvocationResult invoke(String    id,
                                           character name,
                                           String    resId,
                                           boolean   function,
                                           boolean   dynamicFunction,
                                           boolean   superCall,
                                           boolean   transactionDistinct,
                                           String    modes,
                                           Object... args);

   /**
    * Delete the remote procedure, specified by the resource ID. This must be called at the same
    * time when the local proxy procedure is deleted.
    * 
    * @param    id
    *           The appserver connection id. 
    * @param    resId
    *           The ID of a remote procedure which needs to be deleted.
    *           
    * @return   <code>true</code> if the remote procedure was deleted.
    */
   public boolean deleteProcedure(String id, String resId);

   /**
    * Get the name of the remote procedure, specified by the resource ID.
    *  
    * @param    id
    *           The appserver connection id. 
    * @param    resId
    *           The resource ID of a remote procedure.
    *
    * @return   The procedure's name.
    */
   public character getProcedureName(String id, String resId);
}
