/*
** Module   : AppServerInvocationResult.java
** Abstract : Container for the result of a remote appserver procedure call. 
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20130529 Created initial version.
*/

package com.goldencode.p2j.main;

import java.io.*;

import com.goldencode.p2j.util.*;

/**
 * This class acts as a container for the various data which is returned by or updated during 
 * remote appserver calls.  This includes the function/procedure return value, updated 
 * OUTPUT/INPUT-OUTPUT parameters or the persistent procedure handle for a RUN ... PERSISTENT
 * call.
 */
public class AppServerInvocationResult
implements Externalizable
{
   /** The updated parameters, after the remote call. */
   private Object[] arguments = new Object[0];
   
   /** The function/procedure call result. */
   private Object result;

   /** The remote procedure handle, used to build a {@link ProxyProcedureWrapper} instance. */
   private String procedureId;

   /** The error thrown during the remote call or connection request. */
   private Throwable error;
   
   /** The connection ID, returned during the connect request. */
   private String connectionID;
   
   /**
    * Serialize this object, to be picked up by the other side.
    *  
    * @param    out
    *           The output destination to which the data will be sent.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   public void writeExternal(ObjectOutput out)
   throws IOException
   {
      out.writeInt(arguments.length);
      for (int i = 0; i < arguments.length; i++)
      {
         out.writeObject(arguments[i]);
      }
      
      out.writeObject(result);
      out.writeObject(procedureId);
      out.writeObject(error);
      out.writeObject(connectionID);
   }

   /**
    * Used the specified input source and initialize this instance. 
    *  
    * @param    in
    *           The input source from which the data will be restored.
    *
    * @throws   IOException
    *           In case of I/O errors.
    * @throws   ClassNotFoundException
    *           If payload can't be instantiated.
    */
   public void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      int parameterCount = in.readInt();
      arguments = new Object[parameterCount];
      for (int i = 0; i < parameterCount; i++)
      {
         arguments[i] = in.readObject();
      }

      result = (BaseDataType) in.readObject();
      procedureId = (String) in.readObject();
      error = (Throwable) in.readObject();
      connectionID = (String) in.readObject();
   }

   /**
    * Get the returned result, after a function or procedure invocation.
    * 
    * @return   The {@link #result} field.
    */
   public Object getResult()
   {
      return result;
   }

   /**
    * Set the result of this remote call.
    * 
    * @param    result
    *           The result of this remote invocation.
    */
   public void setResult(Object result)
   {
      this.result = result;
   }

   /**
    * Get the handle code for the remote persistent procedure.
    * 
    * @return   The {@link #procedureId} field.
    */
   public String getProcedureId()
   {
      return procedureId;
   }

   /**
    * Save the remote persistent procedure handle's code.
    * 
    * @param    procedureId
    *           The code for the remote persistent procedure handle.
    */
   public void setProcedureId(String procedureId)
   {
      this.procedureId = procedureId;
   }

   /**
    * Get any error thrown by the last remote request.
    * 
    * @return   The {@link #error} field.
    */
   public Throwable getError()
   {
      return this.error;
   }

   /**
    * Set the error thrown by the current remote request.
    * 
    * @param    error
    *           The caught error.
    */
   public void setError(Throwable error)
   {
      this.error = error;
   }

   /**
    * Get the established connection ID, for a connect request.
    * 
    * @return   See above.
    */
   public String getConnectionID()
   {
      return connectionID;
   }

   /**
    * Set the new connection ID, after a connect request.
    * 
    * @param    connectionID
    *           The connection ID, which will be unknown if there are any {@link #error errors}.
    */
   public void setConnectionID(String connectionID)
   {
      this.connectionID = connectionID;
   }

   /**
    * Getter for the {@link #arguments} field.
    * 
    * @return   See above.
    */
   public Object[] getArguments()
   {
      return arguments;
   }

   /**
    * Setter for the {@link #arguments} field.
    * 
    * @param    arguments
    *           The updated arguments, to sent back to the caller.
    */
   public void setArguments(Object[] arguments)
   {
      this.arguments = arguments;
   }
}
