/*
** Module   : TableResultSet.java
** Abstract : A result set used to serialize a table data and send it to a remote side. 
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- ---------------------------------Description----------------------------------
** 001 CA  20130610 Created initial version.
*/

package com.goldencode.p2j.persist;

import java.io.*;
import java.util.*;

/**
 * Provides the main mechanism of passing table-like metadata and data which the remote side will
 * interpret as a temporary table. The remote side will use the sent properties (and their other
 * settings, as type and extent) to dynamically build the DMO interface, implementation class and
 * .hbm.xml configuration file.
 */
public class TableResultSet
implements Externalizable
{
   /**
    * Marker to inform the reader that properties are starting, when deserializing this instance.
    */
   private static final String START_PROPERTY = "START_PROPERTIES";

   /**
    * Marker to inform the reader that the list of properties has ended, when deserializing this
    * instance.
    */
   private static final String END_PROPERTY = "END_PROPERTIES";

   /** Marker to inform the reader that data is starting, when deserializing this instance. */
   private static final String START_DATA = "START_DATA";

   /** Marker to inform the reader that data has ended, when deserializing this instance. */
   private static final String END_DATA = "END_DATA";
   
   /** Default value for no-extent properties. */
   private static final int NO_EXTENT = -2;

   /** Map containing the type of each property. */
   protected Map<String, String> types;

   /**
    * Map containing the extent of each property or <code>null</code., if no-extent property is
    * used.
    */
   protected Map<String, Integer> extents;
   
   /** The list of read properties. */
   private List<String> properties;
   
   /** The read data. */
   private List<Object[]> rows;

   /** Flag indicating this table is sent in INPUT or INPUT-OUTPUT mode. */
   private boolean input;

   /** Flag indicating this table is sent in OUTPUT or INPUT-OUTPUT mode. */
   private boolean output;
   
   /** Flag indicating this table is sent in APPEND mode. */
   private boolean append;
   
   /**
    * When not-null, this instance acts as a wrapper for the {@link TableResultSet} referenced
    * by this field; when serializing this instance, the target {@link #resultSet} will be used.
    */
   private TableResultSet resultSet = null;
   
   /**
    * Default c'tor, explicitly added to allow instances of this class to be created on 
    * deserialization. Not for public use.
    */
   public TableResultSet()
   {
   }
   
   /**
    * Create a new instance of this result set.
    * 
    * @param    input
    *           Flag indicating this table is sent in INPUT or INPUT-OUTPUT mode.
    * @param    output
    *           Flag indicating this table is sent in OUTPUT or INPUT-OUTPUT mode.
    * @param    append
    *           Flag indicating this table is sent in APPEND mode.
    */
   public TableResultSet(boolean input, boolean output, boolean append)
   {
      this.input = input;
      this.output = output;
      this.append = append;
   }
   
   /**
    * Wrap a custom result set in a class known to P2J, so the remote side can deserialize it.
    * 
    * @param    resultSet
    *           The result set, which is a subclass of {@link TableResultSet}.
    */
   public TableResultSet(TableResultSet resultSet)
   {
      this.resultSet = resultSet;
   }

   /**
    * Get the type of the given property.  This interrogates the {@link #types} map, but can be
    * overridden by client code, to satisfy custom needs.  It is also used by
    * {@link #writeExternal}, when sending this object to the remote side.
    * <p>
    * The property's type must be one of the P2J's {@link UserType} sub-classes.
    * 
    * @param    property
    *           The property name.
    *           
    * @return   The propery's type.
    */
   public String getPropertyType(String property)
   {
      return types.get(property);
   }
   
   /**
    * Get the extent of the given property.  This interrogates the {@link #extents} map, but can 
    * be overridden by client code, to satisfy custom needs.  It is also used by
    * {@link #writeExternal}, when sending this object to the remote side.
    * <p>
    * If the property has no explicit extent, then a default value of <code>null</code> will be
    * returned.
    * 
    * @param    property
    *           The property name.
    *           
    * @return   The propery's type.
    */
   public Integer getPropertyExtent(String property)
   {
      return extents.get(property);
   }
   
   /**
    * Used in conjunction with {@link #nextProperty()}, it provides a mechanism for subclasses
    * to send the property list in an iterator-like form.
    * <p>
    * Override this method so that it returns <code>false</code> when {@link #nextProperty} has
    * sent all properties.
    * 
    * @return   Default it returns <code>false<code>.
    */
   public boolean hasMoreProperties()
   {
      return false;
   }
   
   /**
    * Used in conjunction with {@link #hasMoreProperties()}, it provides a mechanism for 
    * subclasses to send the property list in an iterator-like form.
    * <p>
    * Override this method so that when {@link #hasMoreProperties} returns <code>true</code>, this
    * will return the name of the next property in the list.
    * 
    * @return   Default it returns <code>null<code>.
    */
   public String nextProperty()
   {
      return null;
   }
   
   /**
    * Used in conjunction with {@link #hasMoreRows()}, it provides a mechanism for subclasses to
    * send the data in an iterator-like form.
    * <p>
    * Override this method so that it returns <code>false</code> when {@link #nextRow} has sent 
    * all data.
    * 
    * @return   Default it returns <code>false<code>.
    */
   public boolean hasMoreRows()
   {
      return false;
   }
   
   /**
    * Used in conjunction with {@link #hasMoreRows()}, it provides a mechanism for  subclasses to
    * send the data in an iterator-like form.
    * <p>
    * Override this method so that when {@link #hasMoreRows} returns <code>true</code>, this will
    * return the next row in the table.
    * 
    * @return   Default it returns <code>null<code>.
    */
   public Object[] nextRow()
   {
      return null;
   }
   
   /**
    * Check if this table is in INPUT or INPUT-OUTPUT mode.
    * 
    * @return   The {@link #input} flag.
    */
   public boolean isInput()
   {
      return input;
   }
   
   /**
    * Check if this table is in OUTPUT or INPUT-OUTPUT mode.
    * 
    * @return   The {@link #output} flag.
    */
   public boolean isOutput()
   {
      return output;
   }
   
   /**
    * Check if this table is in APPEND mode.
    * 
    * @return   The {@link #append} flag.
    */
   public boolean isAppend()
   {
      return append;
   }
   
   /**
    * Provides an iterator over the read {@link #properties}. 
    * 
    * @return   See above.
    */
   public Iterator<String> propertyIterator()
   {
      return (properties == null ? Collections.EMPTY_LIST : properties).iterator();
   }

   /**
    * Provides an iterator over the read {@link #rows}. 
    * 
    * @return   See above.
    */
   public Iterator<Object[]> rowIterator()
   {
      return (rows == null ? Collections.EMPTY_LIST : rows).iterator();
   }

   /**
    * Get the list of read {@link #properties}.
    * 
    * @return   See above.
    */
   public List<String> getProperties()
   {
      return new ArrayList<String>(properties);
   }

   /**
    * Get the list of read {@link #rows}.
    * 
    * @return   See above.
    */
   public List<Object[]> getRows()
   {
      return new ArrayList<Object[]>(rows);
   }

   /**
    * Send the result set (metadata and rows) to the specified output destination.
    * 
    * @param    out
    *           The output destination to which the result set will be sent.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   public final void writeExternal(ObjectOutput out)
   throws IOException
   {
      if (resultSet != null)
      {
         resultSet.writeExternal(out);
         return;
      }
      
      // write modes
      out.writeBoolean(input);
      out.writeBoolean(output);
      out.writeBoolean(append);
      
      // write properties
      out.writeUTF(START_PROPERTY);
      while (hasMoreProperties())
      {
         String property = nextProperty();
         out.writeUTF(property);
         out.writeUTF(getPropertyType(property));
         Integer extent = getPropertyExtent(property);
         out.writeInt(extent == null ? NO_EXTENT : extent.intValue());
      }
      out.writeUTF(END_PROPERTY);
   
      // write data 
      out.writeObject(START_DATA);
      while (hasMoreRows())
      {
         Object[] row = nextRow();
         out.writeObject(row);
      }
      out.writeObject(END_DATA);
   }

   /**
    * Read the result set (metadata and rows) from the specified input source.
    * 
    * @param    in
    *           The input source from which the result set will be read.
    *
    * @throws   IOException
    *           In case of I/O errors.
    */
   public final void readExternal(ObjectInput in)
   throws IOException,
          ClassNotFoundException
   {
      // read modes
      input = in.readBoolean();
      output = in.readBoolean();
      append = in.readBoolean();
   
      // read columns
      String start = in.readUTF();
      assert (start.equals(START_PROPERTY));
   
      properties = new ArrayList<String>();
      types = new HashMap<String, String>();
      extents = new HashMap<String, Integer>();
      do
      {
         String next = in.readUTF();
         if (END_PROPERTY.equals(next))
         {
            break;
         }
         
         properties.add(next);
         types.put(next, in.readUTF());
         int extent = in.readInt();
         extents.put(next, extent == NO_EXTENT ? null : extent);
      }
      while (true);
   
      // read data
      start = (String) in.readObject();
      assert (START_DATA.equals(start));
      rows = new ArrayList<Object[]>();
      do
      {
         Object next = in.readObject();
         if (END_DATA.equals(next))
         {
            break;
         }
         
         Object[] row = (Object[]) next;
         rows.add(row);
      }
      while (true);
   }
}
