/*
** Module   : SessionUtils.java
** Abstract : Defines APIs used to access the SESSION attributes and methods.
**
** Copyright (c) 2013, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 CA  20130112 Created initial version.
** 002 CS  20130214 Added support for SESSION: EXPORT, SERVER-OPERATING-MODE, REMOTE attributes.
** 003 CA  20130221 getType was renamed to getResourceType.
** 004 CA  20130222 Added stubs for more SESSION-related attributes and methods: REMOTE,
**                  PARAMETER. The implementations for EXPORT, SERVER-OPERATING-MODE and REMOVE 
**                  attrs call their ServerImpl counterparts.
** 005 OM  20130304 Refactored isValid and isUnknown of WrappedResource to valid and unknown.
** 006 CA  20130529 Added implementations for FIRST-SERVER, LAST-SERVER, REMOTE, EXPORT and
**                  SERVER-OPERATING-MODE.
*/

package com.goldencode.p2j.util;

import com.goldencode.p2j.ui.LogicalTerminal;
import com.goldencode.proxy.StaticProxy;

/**
 * This class provides implementation for SESSION attributes and methods.
 * Mainly, the APIs implemented here are defined in the {@link CommonSession}
 * interface (or one of its super interfaces) and are not statically 
 * implemented by the classes used by the {@link #asHandle()}.
 */
public class SessionUtils
{
   /**
    * Get a the instance for the SESSION system handle.  Is obtained using
    * a call to the {@link StaticProxy#obtain(Class, Class[])}, using the
    * {@link CommonSession} interface and its methods implemented by these
    * classes: {@link SessionUtils}, {@link EnvironmentOps}, {@link date},
    * {@link LogicalTerminal}, {@link ProcedureManager}.
    * 
    * @return   See above.
    */
   public static handle asHandle()
   {
      Class[] classes = 
      {
         SessionUtils.class,
         EnvironmentOps.class,
         date.class,
         LogicalTerminal.class,
         ProcedureManager.class,
         NumberType.class
      };
      CommonSession proxy = StaticProxy.obtain(CommonSession.class, 
                                               classes);
      return new handle(proxy);
   }
   
   /**
    * Get the type of its associated handle.
    * 
    * @return   Always return the PSEUDO-WIDGET value.
    */
   public static character getResourceType()
   {
      return new character("PSEUDO-WIDGET");
   }
   
   /**
    * Implementation for the {@link WrappedResource#valid()} API.
    * 
    * @return   Always true.
    */
   public static boolean valid()
   {
      return true;
   }
   
   /**
    * Implementation for the {@link WrappedResource#unknown()} API.
    * 
    * @return   Always false.
    */
   public static boolean unknown()
   {
      return false;
   }
   
   /**
    * API needed to implement read-only attribute assignment (a 4GL 
    * "feature").
    * 
    * @param    attribute
    *           The attribute's name.
    *           
    * @see      handle#readOnlyError(handle, String)
    */
   public static void readOnlyError(String attribute)
   {
      handle.readOnlyError(asHandle(), attribute);
   }
   
   /**
    * Indicates if the session is for the context of an appserver.
    * 
    * @return   <code>true</code> if the session is for the context of an appserver.
    */
   public  static logical isRemote()
   {
      return new logical(_isRemote());
   }

   /**
    * Indicates if the session is for the context of an appserver.
    * 
    * @return   <code>true</code> if the session is for the context of an appserver.
    */
   public static boolean _isRemote()
   {
      return AppServerManager.isRemote();
   }
   
   /**
    * Method implementation for getting the SERVER-OPERATING-MODE attribute of the session handle.
    * <p>
    * Possible values:
    * <ul>
    *    <li>State-reset
    *    <li>State-aware
    *    <li>Stateless
    *    <li>State-free
    * </ul>
    * 
    * @return   The corresponding SERVER-OPERATING-MODE attribute of the session handle.
    */
   public static character getServerOperatingMode()
   {
      return AppServerManager.getServerOperatingMode();
   }
   
   /**
    * Method that specifies the remote procedures the client can execute from the AppServer 
    * session.
    * 
    * @param    procList
    *           The list of procedure  and  name-patterns separated by comma.
    *         
    * @return   <code>true</code> if the method succeeded.
    */
   public static logical export(String procList)
   {
      return AppServerManager.export(procList);
   }
   
   /**
    * Method that specifies the remote procedures the client can execute from the AppServer 
    * session.
    * 
    * @param    procList
    *           The list of procedure  and  name-patterns separated by comma.
    *         
    * @return   <code>true</code> if the method succeeded.
    */
   public static logical export(character procList)
   {
      return AppServerManager.export(procList);
   }

   /**
    * Get the value of the PARAMETER attribute, which is set to the value of the Parameter 
    * (-param) startup parameter specified for the current session.
    * 
    * @return   See above.
    */
   public static character getParameter()
   {
      // TODO: implement this
      return null;
   }

   /**
    * Returns the handle for the first dynamic buffer in the first table containing a dynamic 
    * buffer.
    * <p>
    * TODO: this should call a worker method from the persist package.
    * 
    * @return   See above.
    */
   public static handle firstBuffer()
   {
      // TODO: implement this
      return null;
   }
   
   /**
    * Get the FIRST-SERVER attribute of this session.
    * 
    * @return   See above.
    */
   public static handle firstServer()
   {
      return HandleChain.firstResource(ServerImpl.class);
   }
   
   /**
    * Get the LAST-SERVER attribute of this session.
    * 
    * @return   See above.
    */
   public static handle lastServer()
   {
      return HandleChain.lastResource(ServerImpl.class);
   }
}
